@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2007 daniel g. siegel <dgsiegel@gmail.com>
- * Copyright (C) 2007 Jaap Haitsma <jaap@haitsma.org>
+ * Copyright (C) 2007,2008 daniel g. siegel <dgsiegel@gmail.com>
+ * Copyright (C) 2007,2008 Jaap Haitsma <jaap@haitsma.org>
  * Copyright (C) 2008 Patryk Zawadzki <patrys@pld-linux.org>
  *
  * Licensed under the GNU General Public License Version 2
@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <cheese-config.h>
+#include "cheese-config.h"
 #endif
 
 #include <string.h>
@@ -34,7 +34,6 @@
 #include <gst/interfaces/xoverlay.h>
 #include <gtk/gtk.h>
 #include <libebook/e-book.h>
-#include <glade/glade.h>
 
 #include "cheese-countdown.h"
 #include "cheese-effect-chooser.h"
@@ -45,8 +44,7 @@
 #include "ephy-spinner.h"
 #include "gst-audio-play.h"
 
-#define GLADE_FILE PACKAGE_DATADIR"/cheese.glade"
-#define UI_FILE PACKAGE_DATADIR"/cheese-ui.xml"
+
 #define SHUTTER_SOUNDS 5
 
 typedef enum
@@ -117,48 +115,51 @@ typedef struct
 
 } CheeseWindow;
 
-static void 
-open_url(const char *url, GdkScreen *screen)
-{
-  GError *error = NULL;
-  gboolean ret;
-  char *cmdline;
-  GtkWidget *error_dialog;
-
-  cmdline = g_strconcat ("xdg-open ", url, NULL);
-  ret = gdk_spawn_command_line_on_screen (screen, cmdline, &error);
-  g_free (cmdline);
-
-  if (ret == FALSE) {
-    error_dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, 
-		 	  		   GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
-				           _("Failed to show url %s"), error->message); 
-		                           gtk_dialog_run (GTK_DIALOG (error_dialog));
-    g_error_free(error);
-  }
-}
 
 /* Make url in about dialog clickable */
 static void
 cheese_about_dialog_handle_url (GtkAboutDialog *dialog, const char *url, gpointer data)
 {
-  GdkScreen *gscreen;
-  
-  gscreen = gtk_window_get_screen (GTK_WINDOW (dialog));
-  open_url (url, gscreen);
+  GError *error = NULL;
+  GtkWidget *error_dialog;
+  gboolean ret;
+
+  ret = g_app_info_launch_default_for_uri (url, NULL, &error);
+  if (ret == FALSE)
+  {
+    error_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog), 
+                                           GTK_DIALOG_DESTROY_WITH_PARENT, 
+                                           GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
+                                           _("Failed to open browser to show:\n%s"), url); 
+    gtk_dialog_run (GTK_DIALOG (error_dialog));
+    gtk_widget_destroy (error_dialog);
+    g_error_free(error);
+  }
 }
 
 /* Make email in about dialog clickable */
 static void
 cheese_about_dialog_handle_email (GtkAboutDialog *dialog, const char *email, gpointer data)
 {
-  char *url;
-  GdkScreen *gscreen;
+  char *uri;
+  GError *error = NULL;
+  GtkWidget *error_dialog;
+  gboolean ret;
 
-  gscreen = gtk_window_get_screen (GTK_WINDOW (dialog));
-  url = g_strconcat ("mailto:", email, NULL);
-  open_url (url, gscreen);
-  g_free (url);
+  uri = g_strconcat ("mailto:", email, NULL);
+
+  ret = g_app_info_launch_default_for_uri (uri, NULL, &error);
+  if (ret == FALSE)
+  {
+    error_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog), 
+                                           GTK_DIALOG_DESTROY_WITH_PARENT, 
+                                           GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
+                                           _("Failed to open email client to send message to:\n%s"), email); 
+    gtk_dialog_run (GTK_DIALOG (error_dialog));
+    gtk_widget_destroy (error_dialog);
+    g_error_free(error);
+  }
+  g_free (uri);
 }
 
 static char *
@@ -230,16 +231,26 @@ cheese_window_cmd_open (GtkWidget *widget, CheeseWindow *cheese_window)
 {
   char *uri;
   char *filename;
-  GdkScreen *gscreen;
-
-  gscreen = gtk_window_get_screen (GTK_WINDOW (cheese_window->window));
+  gboolean ret;
+  GError *error = NULL;
+  GtkWidget *dialog;
 
   filename = cheese_thumb_view_get_selected_image (CHEESE_THUMB_VIEW (cheese_window->thumb_view));
   g_return_if_fail (filename);
-
   uri = g_filename_to_uri (filename, NULL, NULL);
   g_free (filename);
-  open_url (uri, gscreen);
+ 
+  ret = g_app_info_launch_default_for_uri (uri, NULL, &error);
+  if (ret == FALSE)
+  {
+    dialog = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window), 
+                                     GTK_DIALOG_DESTROY_WITH_PARENT, 
+                                     GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
+                                     _("Failed to launch program to show:\n%s"), uri);
+    gtk_dialog_run (GTK_DIALOG (dialog));
+    gtk_widget_destroy (dialog);
+    g_error_free(error);
+  }
   g_free (uri);
 }
 
@@ -317,17 +328,19 @@ cheese_window_cmd_move_file_to_trash (CheeseWindow *cheese_window, GList *files)
   gchar *primary, *secondary;
   GtkWidget *error_dialog;
 
-  for (l = files; l != NULL; l = l->next) {
-    if (!g_file_trash (l->data, NULL, &error)) {
+  for (l = files; l != NULL; l = l->next)
+  {
+    if (!g_file_trash (l->data, NULL, &error))
+    {
       primary = g_strdup (_("Cannot move file to trash"));
       secondary = g_strdup_printf (_("The file \"%s\" cannot be moved to the trash. Details: %s"),
-				   g_file_get_basename (l->data), error->message);
+                                   g_file_get_basename (l->data), error->message);
 
       error_dialog = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window),
- 					     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-				             GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, primary);
+                                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                             GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, primary);
       gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
-					        secondary);
+                                                secondary);
       gtk_dialog_run (GTK_DIALOG (error_dialog));
       gtk_widget_destroy (error_dialog);
 
@@ -513,50 +526,21 @@ static void
 cheese_window_cmd_help_contents (GtkAction *action, CheeseWindow *cheese_window)
 {
   GError *error = NULL;
-  char *command;
-  const char *lang;
-  char *uri = NULL;
-  GdkScreen *gscreen;
-  // TODO what do we want for link_id
-  char *link_id = NULL;
+  gboolean ret;
 
-  int i;
+  ret = g_app_info_launch_default_for_uri ("ghelp:cheese", NULL, &error);
 
-  const char * const * langs = g_get_language_names ();
-
-  for (i = 0; langs[i]; i++) 
-  {
-    lang = langs[i];
-    if (strchr (lang, '.')) 
-      continue;
-
-    uri = g_build_filename(DATADIR, "/gnome/help/cheese/", lang, "/cheese.xml", NULL);
-
-    if (g_file_test (uri, G_FILE_TEST_EXISTS))
-      break;
-  }
-
-  if (link_id)
-    command = g_strconcat ("gnome-open ghelp://", uri, "?", link_id, NULL);
-  else
-    command = g_strconcat ("gnome-open ghelp://", uri,  NULL);
-
-
-  gscreen = gdk_screen_get_default();
-  gdk_spawn_command_line_on_screen (gscreen, command, &error);
-  if (error != NULL) 
+  if (ret == FALSE) 
   {
     GtkWidget *d;
-
-    d = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                               GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, error->message);
+    d = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window), 
+                                GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
+                                _("Unable to open help file for Cheese"));
     gtk_dialog_run(GTK_DIALOG(d));
     gtk_widget_destroy(d);
     g_error_free(error);
   }
-
-  g_free (command);
-  g_free (uri);
 }
 
 static void
@@ -573,25 +557,24 @@ cheese_window_cmd_about (GtkAction *action, CheeseWindow *cheese_window)
 
   const char *license[] = {
     N_("This program is free software; you can redistribute it and/or modify "
-        "it under the terms of the GNU General Public License as published by "
-        "the Free Software Foundation; either version 2 of the License, or "
-        "(at your option) any later version.\n"),
+       "it under the terms of the GNU General Public License as published by "
+       "the Free Software Foundation; either version 2 of the License, or "
+       "(at your option) any later version.\n"),
     N_("This program is distributed in the hope that it will be useful, "
-        "but WITHOUT ANY WARRANTY; without even the implied warranty of "
-        "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
-        "GNU General Public License for more details.\n"),
+       "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+       "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
+       "GNU General Public License for more details.\n"),
     N_("You should have received a copy of the GNU General Public License "
-        "along with this program. If not, see <http://www.gnu.org/licenses/>.")
+       "along with this program. If not, see <http://www.gnu.org/licenses/>.")
   };
 
   char *license_trans;
 
-  license_trans = g_strconcat (_(license[0]), "\n", _(license[1]), "\n",
-      _(license[2]), "\n", NULL);
+  license_trans = g_strconcat (_(license[0]), "\n", _(license[1]), "\n", _(license[2]), "\n", NULL);
 
   gtk_show_about_dialog (GTK_WINDOW (cheese_window->window),
                          "version", VERSION,
-                         "copyright", "Copyright \xc2\xa9 2007\n daniel g. siegel <dgsiegel@gmail.com>",
+                         "copyright", "Copyright \xc2\xa9 2007, 2008\n daniel g. siegel <dgsiegel@gmail.com>",
                          "comments", _("A cheesy program to take photos and videos from your webcam"),
                          "authors", authors,
                          "translator-credits", translators,
@@ -676,17 +659,28 @@ cheese_window_effect_button_pressed_cb (GtkWidget *widget, CheeseWindow *cheese_
     gtk_notebook_set_current_page (GTK_NOTEBOOK(cheese_window->notebook), 0);
     gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_effects), _("_Effects"));
     gtk_widget_set_sensitive (cheese_window->take_picture, TRUE);
+    if (cheese_window->webcam_mode == WEBCAM_MODE_PHOTO)
+    {
+      gtk_action_group_set_sensitive (cheese_window->actions_photo, TRUE);
+    }
+    else
+    {
+      gtk_action_group_set_sensitive (cheese_window->actions_video, TRUE);
+    }
     cheese_webcam_set_effect (cheese_window->webcam, 
                               cheese_effect_chooser_get_selection (CHEESE_EFFECT_CHOOSER (cheese_window->effect_chooser)));
     g_object_set (cheese_window->gconf,
-                 "gconf_prop_selected_effects", cheese_effect_chooser_get_selection_string (CHEESE_EFFECT_CHOOSER (cheese_window->effect_chooser)),
-                 NULL);
+                  "gconf_prop_selected_effects",
+                  cheese_effect_chooser_get_selection_string (CHEESE_EFFECT_CHOOSER (cheese_window->effect_chooser)),
+                  NULL);
   }
   else
   {
     gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->notebook), 1);
     gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_effects), _("_Back"));
     gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture), FALSE);
+    gtk_action_group_set_sensitive (cheese_window->actions_photo, FALSE);
+    gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
   }
 }
 
@@ -710,7 +704,8 @@ cheese_window_photo_video_toggle_buttons_cb (GtkWidget *widget, CheeseWindow *ch
   // FIXME: THIS IS CRAP!
   actions = gtk_action_group_list_actions (cheese_window->actions_toggle);
   tmp = actions;
-  while (tmp != NULL) {
+  while (tmp != NULL)
+  {
     if (strcmp (gtk_action_get_name (GTK_ACTION (tmp->data)), "Photo") == 0)
       photo = tmp->data;
     else
@@ -757,6 +752,8 @@ cheese_window_photo_video_toggle_buttons_cb (GtkWidget *widget, CheeseWindow *ch
     g_free (str);
     gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
     gtk_action_activate (video);
+    gtk_action_group_set_sensitive (cheese_window->actions_photo, FALSE);
+    gtk_action_group_set_sensitive (cheese_window->actions_video, TRUE);
   }
   else
   {
@@ -767,6 +764,8 @@ cheese_window_photo_video_toggle_buttons_cb (GtkWidget *widget, CheeseWindow *ch
     g_free (str);
     gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
     gtk_action_activate (photo);
+    gtk_action_group_set_sensitive (cheese_window->actions_photo, TRUE);
+    gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
   }
   g_list_free (actions);
   g_list_free (tmp);
@@ -802,6 +801,45 @@ cheese_window_countdown_picture_cb (gpointer data)
   cheese_webcam_take_photo (cheese_window->webcam, cheese_window->photo_filename);
 }
 
+static void
+cheese_window_stop_recording (CheeseWindow *cheese_window)
+{
+  if (cheese_window->recording)
+  {
+    cheese_webcam_stop_video_recording (cheese_window->webcam);
+    gtk_widget_set_sensitive (cheese_window->button_effects, TRUE);
+    gtk_widget_set_sensitive (cheese_window->button_photo, TRUE);
+    gtk_widget_set_sensitive (cheese_window->button_video, TRUE);
+    gchar * str = g_strconcat ("<b>", _("_Start recording"), "</b>", NULL);
+    gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
+    g_free (str);
+    gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
+    gtk_image_set_from_stock (GTK_IMAGE (cheese_window->image_take_photo), GTK_STOCK_MEDIA_RECORD, GTK_ICON_SIZE_BUTTON);
+    
+    cheese_window->recording = FALSE;
+  }
+}
+
+static gboolean
+cheese_window_cancel_cb (CheeseWindow *cheese_window,
+                         GtkAccelGroup *accel_group,
+                         guint keyval, GdkModifierType modifier)
+{
+  cheese_countdown_cancel ((CheeseCountdown *) cheese_window->countdown);
+  
+  if (cheese_window->webcam_mode == WEBCAM_MODE_PHOTO)
+  {
+    gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->notebook_bar), 0);
+      
+    gtk_widget_set_sensitive (cheese_window->take_picture, TRUE);
+  }
+  else
+  {
+    cheese_window_stop_recording (cheese_window);
+  }
+  return TRUE;
+}
+
 static void
 cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_window)
 {
@@ -809,8 +847,20 @@ cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_
 
   if (cheese_window->webcam_mode == WEBCAM_MODE_PHOTO)
   {
-    cheese_countdown_start((CheeseCountdown *) cheese_window->countdown, cheese_window_countdown_picture_cb, cheese_window_countdown_hide_cb, (gpointer) cheese_window);
-    gtk_notebook_set_current_page (GTK_NOTEBOOK(cheese_window->notebook_bar), 1);
+    gboolean countdown;
+    g_object_get (cheese_window->gconf, "gconf_prop_countdown", &countdown, NULL);
+    if (countdown)
+    {
+      cheese_countdown_start((CheeseCountdown *) cheese_window->countdown,
+                             cheese_window_countdown_picture_cb,
+                             cheese_window_countdown_hide_cb,
+                             (gpointer) cheese_window);
+      gtk_notebook_set_current_page (GTK_NOTEBOOK(cheese_window->notebook_bar), 1);
+    }
+    else
+    {
+      cheese_window_countdown_picture_cb (cheese_window);
+    }
 
     gtk_widget_set_sensitive (cheese_window->take_picture, FALSE);
     // FIXME: set menu inactive
@@ -830,20 +880,13 @@ cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_
 
       cheese_window->video_filename = cheese_fileutil_get_new_media_filename (WEBCAM_MODE_VIDEO);
       cheese_webcam_start_video_recording (cheese_window->webcam, cheese_window->video_filename);
+
+      cheese_window->recording = TRUE;
     }
     else
     {
-      cheese_webcam_stop_video_recording (cheese_window->webcam);
-      gtk_widget_set_sensitive (cheese_window->button_effects, TRUE);
-      gtk_widget_set_sensitive (cheese_window->button_photo, TRUE);
-      gtk_widget_set_sensitive (cheese_window->button_video, TRUE);
-      str = g_strconcat ("<b>", _("_Start recording"), "</b>", NULL);
-      gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
-      g_free (str);
-      gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
-      gtk_image_set_from_stock (GTK_IMAGE (cheese_window->image_take_photo), GTK_STOCK_MEDIA_RECORD, GTK_ICON_SIZE_BUTTON);
+      cheese_window_stop_recording (cheese_window);
     }
-    cheese_window->recording = !cheese_window->recording;
   }
 }
 
@@ -903,7 +946,8 @@ static const GtkActionEntry action_entries_flickr[] = {
 static void
 cheese_window_activate_radio_action (GtkAction *action, GtkRadioAction *current, CheeseWindow *cheese_window)
 {
-  if (strcmp (gtk_action_get_name (GTK_ACTION (current)), "Photo") == 0) {
+  if (strcmp (gtk_action_get_name (GTK_ACTION (current)), "Photo") == 0)
+  {
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cheese_window->button_photo), TRUE);
   }
   else
@@ -962,51 +1006,61 @@ cheese_window_radio_action_group_new (CheeseWindow *cheese_window, char *name,
 static void
 cheese_window_create_window (CheeseWindow *cheese_window)
 {
-  GladeXML *gxml;
   GtkWidget *menubar;
   GError *error=NULL;
   char *path;
+  GtkBuilder* builder;
 
-  gxml = glade_xml_new (GLADE_FILE, NULL, NULL);
-  cheese_window->window              = glade_xml_get_widget (gxml, "cheese_window");
-  cheese_window->button_effects      = glade_xml_get_widget (gxml, "button_effects");
-  cheese_window->button_photo        = glade_xml_get_widget (gxml, "button_photo");
-  cheese_window->button_video        = glade_xml_get_widget (gxml, "button_video");
-  cheese_window->image_take_photo    = glade_xml_get_widget (gxml, "image_take_photo");
-  cheese_window->label_effects       = glade_xml_get_widget (gxml, "label_effects");
-  cheese_window->label_photo         = glade_xml_get_widget (gxml, "label_photo");
-  cheese_window->label_take_photo    = glade_xml_get_widget (gxml, "label_take_photo");
-  cheese_window->label_video         = glade_xml_get_widget (gxml, "label_video");
-  cheese_window->main_vbox           = glade_xml_get_widget (gxml, "main_vbox");
-  cheese_window->notebook            = glade_xml_get_widget (gxml, "notebook");
-  cheese_window->notebook_bar        = glade_xml_get_widget (gxml, "notebook_bar");
-  cheese_window->screen              = glade_xml_get_widget (gxml, "video_screen");
-  cheese_window->take_picture        = glade_xml_get_widget (gxml, "take_picture");
+  builder = gtk_builder_new ();
+  gtk_builder_add_from_file (builder, PACKAGE_DATADIR"/cheese.ui", &error);
+
+  if (error)
+  {
+    g_error ("building ui from %s failed: %s", PACKAGE_DATADIR"/cheese.ui", error->message);
+    g_clear_error (&error);
+  }
+
+  cheese_window->window             = GTK_WIDGET (gtk_builder_get_object (builder, "cheese_window"));
+  cheese_window->button_effects     = GTK_WIDGET (gtk_builder_get_object (builder, "button_effects"));
+  cheese_window->button_photo       = GTK_WIDGET (gtk_builder_get_object (builder, "button_photo"));
+  cheese_window->button_video       = GTK_WIDGET (gtk_builder_get_object (builder, "button_video"));
+  cheese_window->image_take_photo   = GTK_WIDGET (gtk_builder_get_object (builder, "image_take_photo"));
+  cheese_window->label_effects      = GTK_WIDGET (gtk_builder_get_object (builder, "label_effects"));
+  cheese_window->label_photo        = GTK_WIDGET (gtk_builder_get_object (builder, "label_photo"));
+  cheese_window->label_take_photo   = GTK_WIDGET (gtk_builder_get_object (builder, "label_take_photo"));
+  cheese_window->label_video        = GTK_WIDGET (gtk_builder_get_object (builder, "label_video"));
+  cheese_window->main_vbox          = GTK_WIDGET (gtk_builder_get_object (builder, "main_vbox"));
+  cheese_window->notebook           = GTK_WIDGET (gtk_builder_get_object (builder, "notebook"));
+  cheese_window->notebook_bar       = GTK_WIDGET (gtk_builder_get_object (builder, "notebook_bar"));
+  cheese_window->screen             = GTK_WIDGET (gtk_builder_get_object (builder, "video_screen"));
+  cheese_window->take_picture       = GTK_WIDGET (gtk_builder_get_object (builder, "take_picture"));
+  cheese_window->thumb_scrollwindow = GTK_WIDGET (gtk_builder_get_object (builder, "thumb_scrollwindow"));
+  cheese_window->throbber_frame     = GTK_WIDGET (gtk_builder_get_object (builder, "throbber_frame"));
+  cheese_window->countdown_frame    = GTK_WIDGET (gtk_builder_get_object (builder, "countdown_frame"));
+  cheese_window->effect_frame       = GTK_WIDGET (gtk_builder_get_object (builder, "effect_frame"));
+
+  g_object_unref (builder);
 
   char *str = g_strconcat ("<b>", _("_Take a photo"), "</b>", NULL);
   gtk_label_set_text_with_mnemonic (GTK_LABEL (cheese_window->label_take_photo), str);
   g_free (str);
   gtk_label_set_use_markup (GTK_LABEL (cheese_window->label_take_photo), TRUE);
 
-  cheese_window->thumb_scrollwindow  = glade_xml_get_widget (gxml, "thumb_scrollwindow");
-  cheese_window->thumb_view          = cheese_thumb_view_new ();
+  cheese_window->thumb_view = cheese_thumb_view_new ();
   gtk_container_add (GTK_CONTAINER (cheese_window->thumb_scrollwindow), cheese_window->thumb_view);
 
   char *gconf_effects;
   g_object_get (cheese_window->gconf, "gconf_prop_selected_effects", &gconf_effects, NULL);
-  cheese_window->effect_frame        = glade_xml_get_widget (gxml, "effect_frame");
-  cheese_window->effect_chooser      = cheese_effect_chooser_new (gconf_effects);
+  cheese_window->effect_chooser = cheese_effect_chooser_new (gconf_effects);
   gtk_container_add (GTK_CONTAINER (cheese_window->effect_frame), cheese_window->effect_chooser);
   g_free (gconf_effects);
 
-  cheese_window->throbber_frame      = glade_xml_get_widget (gxml, "throbber_frame");
-  cheese_window->throbber            = ephy_spinner_new ();
+  cheese_window->throbber = ephy_spinner_new ();
   ephy_spinner_set_size (EPHY_SPINNER (cheese_window->throbber), GTK_ICON_SIZE_DIALOG);
   gtk_container_add (GTK_CONTAINER (cheese_window->throbber_frame), cheese_window->throbber);
   gtk_widget_show (cheese_window->throbber);
 
-  cheese_window->countdown_frame      = glade_xml_get_widget (gxml, "countdown_frame");
-  cheese_window->countdown            = cheese_countdown_new ();
+  cheese_window->countdown = cheese_countdown_new ();
   gtk_container_add (GTK_CONTAINER (cheese_window->countdown_frame), cheese_window->countdown);
   gtk_widget_show (cheese_window->countdown);
 
@@ -1027,15 +1081,15 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                                                                 action_entries_main, 
                                                                 G_N_ELEMENTS (action_entries_main));
   cheese_window->actions_toggle = cheese_window_radio_action_group_new (cheese_window, 
-                                                                "ActionsRadio", 
-                                                                action_entries_toggle, 
-                                                                G_N_ELEMENTS (action_entries_toggle));
+                                                                        "ActionsRadio", 
+                                                                        action_entries_toggle, 
+                                                                        G_N_ELEMENTS (action_entries_toggle));
   cheese_window->actions_effects = cheese_window_toggle_action_group_new (cheese_window, 
-                                                                "ActionsMain", 
-                                                                action_entries_effects, 
-                                                                G_N_ELEMENTS (action_entries_effects));
+                                                                          "ActionsEffects", 
+                                                                          action_entries_effects, 
+                                                                          G_N_ELEMENTS (action_entries_effects));
   cheese_window->actions_file = cheese_window_action_group_new (cheese_window, 
-                                                                "ActionsMain", 
+                                                                "ActionsFile", 
                                                                 action_entries_edit_file, 
                                                                 G_N_ELEMENTS (action_entries_edit_file));
   cheese_window->actions_photo = cheese_window_action_group_new (cheese_window, 
@@ -1043,9 +1097,9 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                                                                  action_entries_photo, 
                                                                  G_N_ELEMENTS (action_entries_photo));
   cheese_window->actions_video = cheese_window_toggle_action_group_new (cheese_window, 
-                                                                 "ActionsVideo", 
-                                                                 action_entries_video, 
-                                                                 G_N_ELEMENTS (action_entries_video));
+                                                                        "ActionsVideo", 
+                                                                        action_entries_video, 
+                                                                        G_N_ELEMENTS (action_entries_video));
   gtk_action_group_set_sensitive (cheese_window->actions_video, FALSE);
   cheese_window->actions_account_photo = cheese_window_action_group_new (cheese_window, 
                                                                          "ActionsAccountPhoto", 
@@ -1076,11 +1130,11 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   gtk_action_group_set_visible (cheese_window->actions_flickr, path != NULL);
   g_free (path);
 
-  gtk_ui_manager_add_ui_from_file (cheese_window->ui_manager, UI_FILE, &error);
+  gtk_ui_manager_add_ui_from_file (cheese_window->ui_manager, PACKAGE_DATADIR"/cheese-ui.xml", &error);
 
   if (error)
   {
-    g_critical ("building menus from %s failed: %s", UI_FILE, error->message);
+    g_critical ("building menus from %s failed: %s", PACKAGE_DATADIR"/cheese-ui.xml", error->message);
     g_error_free (error);
   }
 
@@ -1092,20 +1146,23 @@ cheese_window_create_window (CheeseWindow *cheese_window)
 
   gtk_window_add_accel_group (GTK_WINDOW (cheese_window->window), 
                               gtk_ui_manager_get_accel_group (cheese_window->ui_manager));
+  gtk_accel_group_connect (gtk_ui_manager_get_accel_group (cheese_window->ui_manager),
+                           GDK_Escape, 0, 0,
+                           g_cclosure_new_swap (G_CALLBACK (cheese_window_cancel_cb),
+                                                cheese_window, NULL));
+
   gtk_action_group_set_sensitive (cheese_window->actions_file, FALSE);
 
   /* Default handlers for closing the application */
   g_signal_connect (cheese_window->window, "destroy",
                     G_CALLBACK (cheese_window_cmd_close), cheese_window);
-  g_signal_connect(cheese_window->window, "delete_event", 
-                   G_CALLBACK(cheese_window_delete_event_cb), NULL);
+  g_signal_connect (cheese_window->window, "delete_event", 
+                    G_CALLBACK(cheese_window_delete_event_cb), NULL);
 
   g_signal_connect (cheese_window->take_picture, "clicked",
                     G_CALLBACK (cheese_window_action_button_clicked_cb), cheese_window);
-
   g_signal_connect (cheese_window->button_effects,
                     "clicked", G_CALLBACK (cheese_window_effect_button_pressed_cb), cheese_window);
-
   g_signal_connect (cheese_window->thumb_view, "button_press_event",
                     G_CALLBACK (cheese_window_button_press_event_cb), cheese_window);
 }
@@ -1129,8 +1186,10 @@ setup_camera (CheeseWindow *cheese_window)
                             cheese_effect_chooser_get_selection (CHEESE_EFFECT_CHOOSER (cheese_window->effect_chooser)));
 
   cheese_webcam_play (cheese_window->webcam);
+  gdk_threads_enter ();
   gtk_notebook_set_current_page (GTK_NOTEBOOK(cheese_window->notebook), 0);
   ephy_spinner_stop (EPHY_SPINNER (cheese_window->throbber));
+  gdk_threads_leave ();
 }
 
 void
@@ -1156,7 +1215,8 @@ cheese_window_init ()
   
   /* Run cam setup in its own thread */
   GError *error = NULL;
-  if (!g_thread_create ((GThreadFunc) setup_camera, cheese_window, FALSE, &error)) {
+  if (!g_thread_create ((GThreadFunc) setup_camera, cheese_window, FALSE, &error))
+  {
     g_error ("Failed to create setup thread: %s\n", error->message);
     g_error_free (error);
     return;
